﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;

namespace AlliesTests
{
    class AlliesTests
    {
        static void Main(string[] args)
        {
            if (args.Length == 1)
                (new Tests(int.Parse(args[0]))).Run();
            else
            {
                (new Tests()).Run();
            }
        }

        public class Tests
        {
            private struct Data
            {
                public readonly int M;
                public readonly int T;
                public readonly int K;
                public const int Min = 0;
                public const int Max = 1000000000;

                public Data(int m, int t, int k)
                {
                    M = m;
                    T = t;
                    K = k;
                }

                private bool Equals(Data other)
                {
                    return K == other.K && T == other.T && M == other.M;
                }

                public override bool Equals(object obj)
                {
                    if (ReferenceEquals(null, obj)) return false;
                    return obj is Data && Equals((Data)obj);
                }

                public override int GetHashCode()
                {
                    unchecked
                    {
                        var hashCode = K;
                        hashCode = (hashCode * 397) ^ T;
                        hashCode = (hashCode * 397) ^ M;
                        return hashCode;
                    }
                }
            }

            #region Fields
            private readonly StreamWriter _writer;
            private const string FileName = "ALLIES.IN";
            private readonly int _tests = 10000;
            #endregion

            #region Ctor
            public Tests()
            {
                _writer = new StreamWriter(new FileStream(FileName, FileMode.Create));
            }
            public Tests(int tests)
            {
                _writer = new StreamWriter(new FileStream(FileName, FileMode.Create));
                _tests = tests;
            }
            #endregion

            #region Methods

            public void Run()
            {

                var generatedData = GenerateTests();
                _writer.WriteLine(generatedData.Length.ToString());
                PrintTest(generatedData);
                _writer.Close();
            }

            private void PrintTest(Data data)
            {
                _writer.WriteLine("{0} {1} {2}", data.M, data.T, data.K);
            }

            private void PrintTest(IEnumerable<Data> data)
            {
                foreach (var item in data)
                {
                    PrintTest(item);
                }
            }

            private Data[] GenerateTests()
            {
                Random random = new Random(42);
                var defaultData = GetDefaultData();
                var minMaxData = GetMinMaxData();
                var firstData = new Data[defaultData.Length + minMaxData.Length];
                defaultData.CopyTo(firstData, 0);
                minMaxData.CopyTo(firstData, defaultData.Length);
                var generatedData = new HashSet<Data>(firstData);

                while (generatedData.Count < _tests)
                {
                    for (int m = 10; m <= Data.Max; m *= 10)
                    {
                        for (int t = 10; t <= Data.Max; t *= 10)
                        {
                            int M = random.Next(Data.Min, m);
                            int T = random.Next(Data.Min, t);
                            int K = random.Next(Data.Min, Math.Min(M, T));
                            generatedData.Add(new Data(M, T, K));
                            if (generatedData.Count == _tests)
                                return generatedData.ToArray();
                        }
                    }
                }

                return generatedData.ToArray();
            }

            private Data[] GetDefaultData()
            {
                return new[]
                {
                    new Data(3,2,1),
                    new Data(20,23,15),
                    new Data(30,30,30)
                };
            }

            private Data[] GetMinMaxData()
            {
                int[] minMax = { Data.Min, Data.Max };
                var list = new List<Data>();
                foreach (int t1 in minMax)
                {
                    foreach (int t2 in minMax)
                    {
                        list.AddRange(minMax.Select(l => new Data(t1, t2, 0)));
                        list.AddRange(minMax.Select(l => new Data(t1, t2, Math.Min(t1, t2))));
                    }
                }
                return list.ToArray();
            }
            #endregion
        }

    }
}
