#include <iostream>
#include <stdio.h>
#include <vector>
#include <queue>
#include <memory.h>

const int MAX_SIZE_SIDE_FIELD = 101;
const int MAX_COUNT_VERTICES = MAX_SIZE_SIDE_FIELD * MAX_SIZE_SIDE_FIELD * 2;

using namespace std;

struct TEdge
{
	int Start,
		End,
		Id,
		Capacity,
		Flow;
	
	TEdge(int start, int end, int id, int capacity, int flow = 0) :
		Start(start),
		End(end),
		Id(id),
		Capacity(capacity),
		Flow(flow) {}
};

vector<TEdge> Edges;
vector<int> Vertices[MAX_COUNT_VERTICES];
int VerticesOrder[MAX_COUNT_VERTICES];
size_t IndicesInVertices[MAX_COUNT_VERTICES];
int S, T, VerticesCount;

char Field[MAX_SIZE_SIDE_FIELD][MAX_SIZE_SIDE_FIELD];

void addEdge(int from, int to, int capacity)
{
	int id = Edges.size();
	Edges.push_back({from, to, id, capacity});
	Vertices[from].push_back(id);
	
	Edges.push_back({to, from, ++id, 0});
	Vertices[to].push_back(id);
}

bool bfs()
{
	memset(VerticesOrder, -1, VerticesCount * sizeof(VerticesOrder[0]));
	VerticesOrder[S] = 0;
	queue<int> q;
	q.push(S);
	while (!q.empty()) {
		int vertex = q.front();
		q.pop();
		
		for (int indexEdge : Vertices[vertex]) {
			TEdge &edge = Edges[indexEdge];
			if (VerticesOrder[edge.End] == -1 && edge.Flow < edge.Capacity) {
				q.push(edge.End);
				VerticesOrder[edge.End] = VerticesOrder[vertex] + 1;
			}
		}
	}
	return VerticesOrder[T] != -1;
}

int dfs(int currentVertex, int currentFlow)
{
	if (currentVertex == T || !currentFlow)
		return currentFlow;
	for (size_t &indexEdge = IndicesInVertices[currentVertex]; indexEdge < Vertices[currentVertex].size(); ++indexEdge) {
		TEdge &edge = Edges[Vertices[currentVertex][indexEdge]];
		if (VerticesOrder[currentVertex] + 1 == VerticesOrder[edge.End]) {
			int flow = dfs(edge.End, min(currentFlow, edge.Capacity - edge.Flow));
			if (flow) {
				edge.Flow += flow;
				Edges[edge.Id ^ 1].Flow -= flow;
				return flow;
			}
		}
	}
	return 0;
}

int dinic()
{
	int flow = 0;
	while (bfs()) {
		memset(IndicesInVertices, 0, VerticesCount * sizeof(IndicesInVertices[0]));
		while (int add = dfs(S, INT_MAX)) {
			flow += add;
		}
	}
	return flow;
}

int getNumberVertex(int row, int column, int columnCount)
{
	return 2 * (row * columnCount + column);
}

void clear()
{
	for (int i = 0; i < VerticesCount; ++i)
		Vertices[i].clear();
	Edges.clear();
}

int main()
{
	//freopen("input.txt", "r", stdin);
	freopen("bizons.in", "r", stdin); freopen("bizons.out", "w", stdout);
	int testCount;
	int rowCount, columnCount;
	scanf("%d", &testCount);
	while (testCount--) {
		clear();
		scanf("%d%d", &rowCount, &columnCount);
		fflush(stdout);
		for (int i = 0; i < rowCount; ++i)
			scanf("%s", Field[i]);
	
		T = rowCount * columnCount * 2;
		VerticesCount = T + 1;
		for (int row = 0; row < rowCount; ++row) {
			for (int column = 0; column < columnCount; ++column)
				if (Field[row][column] != '#') {
					int number = getNumberVertex(row, column, columnCount);
					if (Field[row][column] == 'C')
						S = number + 1;
					else
						addEdge(number, number + 1, 1); //раздваиваем вершины
				
					if (Field[row][column] == 'S' || Field[row][column] == 'H') {
						addEdge(number + 1, T, 1);
						continue;
					}
				
					if (row && Field[row - 1][column] != '#')
						addEdge(number + 1, getNumberVertex(row - 1, column, columnCount), 1);
				
					if (column && Field[row][column - 1] != '#')
						addEdge(number + 1, getNumberVertex(row, column - 1, columnCount), 1);
				
					if (row + 1 < rowCount && Field[row + 1][column] != '#')
						addEdge(number + 1, getNumberVertex(row + 1, column, columnCount), 1);
				
					if (column + 1 < columnCount && Field[row][column + 1] != '#')
						addEdge(number + 1, getNumberVertex(row, column + 1, columnCount), 1);
				
				}
			
		}

		puts(dinic() == 2 ? "Yes" : "No");
	}
	return 0;
}