#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <iostream>
#include <set>
#include <map>
#include <algorithm>
#include <string>
#include <vector>
#include <fstream>

#define file_name "fairyland"
#define F first
#define S second
#define sqr(a) ((a)*(a))
#define in(x, l, r) (((l)<=(x))&&((x)<(r)))
#define min(a, b) (((a)<(b))?(a):(b))
#define max(a, b) (((a)>(b))?(a):(b))

typedef long long ll;
using namespace std;

const double PI=acos(-1.0);
const double eps = 1e-10;
const int oo = 2e9+1;

//K-размер блока
//M-количество блоков
const int K = 35;
const int N = 2e4+3;
const int M = N/K+3;

//В d[i][j] хранится следующее значение: если блок в котором находится элемент под номером j раньше блока под номером i,
//то в d[i][j] хранится ответ для отрезка начиная с j-го элемента и заканчивая последний элементом i-го блока.
//Но если блок в котором находится элемент j позже i-го блока, то в d[i][j] хранится ответ для отрезка начиная с первого
//элемента i-го блока и заканчивая элементом под номером j.

//В d1[i] хранится ответ для отрезка начинающегося в элементе под номером i и заканчивающимся в последнем элементе блока, в котором находится элемент i.

//В d2[i] хранится ответ для отрезка начинающегося в первом элементе блока где лежит элемент под номером i, и заканчивающегося в элементе под номером i.
 
int d[M][N], d1[N], d2[N], n;
pair <int, int> a[N], b[N];

//Данная функция считает ответ для между блоками под номерами i и j. То есть находит min(abs(A1-A2)), где A1 - элемент из первого блока, а A2 из второго.
//Причем в блоке i берутся те элементы номера которых лежат в отрезке [l1,r1],
//а в блоке j берутся те элементы номера которых лежат в отрезке [l2, r2].
//
//Если i==j, то функция ищет ответ для i-го блока, причем учитывает только те элементы
//номера которых лежат в отрезке [l1, r1].
//
//Время работы O(K), так как элементы в блоках отсортированы и для нахождения ответа используется метод двух указателей.
int get(int i, int j, int l1, int r1, int l2, int r2)
{
    int ans, u, v, pv, pu, t1, t2, li, ri, lj, rj;
    li=i*K; ri=min(li+K, n);
    lj=j*K; rj=min(lj+K, n);
    ans=oo;
    if(i==j)
    {
        for(pu=-1, u=li; u<ri; ++u)
        {
            if(!in(a[u].S, l1, r1)) continue;
            if(pu!=-1) ans=min(ans, abs(a[u].F-a[pu].F));
            pu=u;
        }
    }
    else
    {
        pv=-1;
        for(u=li, v=lj; u<ri; ++u)
        {
            if(!in(a[u].S, l1, r1)) continue;
            for(;v<rj; ++v)
            {
                if(!in(a[v].S, l2, r2)) continue;
                if(a[v].F<=a[u].F)
                {
                    pv=v;
                    continue;
                }
                break;
            }
            t1=t2=oo;
            if(pv!=-1) t1=abs(a[u].F-a[pv].F);
            if(v<rj)   t2=abs(a[u].F-a[v].F);
            ans=min(ans, min(t1, t2));
        }
    }
    return ans;
}

//Функция считает массивы d, d1, d2.
void make()
{
    int i, j, n1, m, u, ym, ln, tmp, ik, v, l, r, l1, r1, t1, t2, l2, r2, t;
    m=(n+K-1)/K;
	
    //Нижний цикл считает массивы d1,d2. Массив a[] пока не отсортирован. 
    //Массивы d1 и d2 можно посчитать за O(M*K*K)=O(N*K) обычным перебором всех вариантом для каждого элемента.
    for(i=0; i<m; ++i)
    {
        l=i*K; r=min(n,l+K);
        for(u=l; u<r; ++u)
        {
            d1[u]=d2[u]=oo;
            for(v=u+1; v<r; ++v) d1[u]=min(d1[u], abs(a[u].F-a[v].F));
            for(v=u-1; v>=l; --v) d2[u]=min(d2[u], abs(a[u].F-a[v].F));
        }
        for(u=r-2; u>=l; --u) d1[u]=min(d1[u], d1[u+1]);
        for(u=l+1; u<r; ++u) d2[u]=min(d2[u], d2[u-1]);
    }
    //Массив b[] это массив отсортированный массив a[].
    //А сам массив a[] сортируется внутри каждого блока.
    for(i=0; i<n; ++i) b[i]=a[i];
    sort(b, b+n);
    for(i=0; i<n; i+=K) sort(a+i, a+min(n, i+K) );
    for(i=0; i<m; ++i)
    {
        l1=i*K; r1=min(n, l1+K);
        //Переменная v бежит по элементам блока i. 
        //А переменная u бежит по всем элементам сразу.
        v=l1;
        for(u=0; u<n; ++u)
        {
            for(;(v<r1)&&(a[v].F<=b[u].F); ++v);
            t1=t2=oo;
            if(v>l1) t1=abs(a[v-1].F-b[u].F);
            if(v<r1) t2=abs(a[v].F-b[u].F);
            d[i][b[u].S]=min(d1[l1], min(t1, t2));
        }
        //Приведу пример, чтобы было понятно, что делает нижележащий цикл:
        //Пусть есть два блока i и j, причем j<i, а так же элемент с номером u из блока j.
        //Пока в d[i][u] лежит значение равное min(abs(A[u]-A[v])), где v это номера всех элементов из i-го блока. 
        //Здесь за A[v] обозначается позиция в еще не отсортированном массиве а[].
        //Цикл после комментария делает следующее: d[i][u]=min(abs(A[l]-A[v])), 
        //где v это номера всех элементов из i-го блока, а l это номера всех элементов из j-го блока, такие что l>=u.  
        for(j=0; j<m; ++j)
        {
            l2=j*K; r2=min(n, l2+K);
            if(i>j) for(u=r2-2; u>=l2; --u) d[i][u]=min(d[i][u], d[i][u+1]);
            if(i<j) for(u=l2+1; u<r2; ++u)  d[i][u]=min(d[i][u], d[i][u-1]);
        }
    }
    //Чтобы понять этот цикл снова возьмем блоки j<i и элемент под номером u из блока j.
    //Теперь в d[i][u] будет положено следующее значение d[i][u]=min(abs(A[l]-A[v])),
    //где v это номера всех элементов начиная с блока j+1 до i-го блока, а l это номера 
    //элементов из блока j, таких что l>=u. 
    for(ln=2; ln<m; ++ln)
    {
        for(i=0; i+ln<m; ++i)
        {
            j=i+ln;
            l=j*K; r=min(l+K, n); t=oo;
            t=d[i][(j-1)*K+K-1];
            for(u=l; u<r; ++u) d[i][u]=min(d[i][u], min(t, d[i+1][u]));
        }
        for(i=m-1; i-ln>=0; --i)
        {
            j=i-ln;
            l=j*K; r=min(l+K, n); t=oo;
            t=d[i][(j+1)*K];
            for(u=l; u<r; ++u) d[i][u]=min(d[i][u], min(t, d[i-1][u]));
        }
    }
    //В комментариях были рассмотрены ситуации когда j<i, в случае когда j>i все почти также;)
}
void answer()
{
    int i, m, l, r, c, ans, j;

    scanf("%d", &n);
    for(i=0; i<n; ++i)
    {
        scanf("%d", &c);
        a[i]=make_pair(c, i);
    }
    make();
    scanf("%d", &m);
    ans=0;
    while(m--)
    {
        scanf("%d%d", &l, &r);
        l=(l+ans)%n;
        r=(r+ans)%n;
        i=l/K;
        j=r/K;
        ans=get(i, j, l, r+1, l, r+1);
        if(i!=j)
        {
            ans=min(ans, d1[l]);
            ans=min(ans, d2[r]);
            if(i+1!=j)
            {
                ans=min(ans, d[j-1][l]);
                ans=min(ans, d[i+1][r]);
            }
        }
        printf("%d\n", ans);
    }
}

int main()
{
    //freopen("f.in", "r", stdin);
    //freopen("output.txt", "w", stdout);
    freopen(file_name".in", "r", stdin); //freopen(file_name".out", "w", stdout);

    int t;
    scanf("%d", &t);
    while(t--) answer();


    return 0;
}

